import psycopg2

from sparweltbitool.config import config
from sparweltbitool.singleton import Singleton

@Singleton
class RedshiftSingleton(object):
    """ Operations with Redshift database."""

    def __init__(self):
        parameters = {
            'host': config.get('database', 'host'),
            'dbname': config.get('database', 'dbname'),
            'user': config.get('database', 'user'),
            'password': config.get('database', 'password'),
            'port': config.get('database', 'port')
        }
        self.conn = psycopg2.connect("host='%(host)s' dbname='%(dbname)s' user='%(user)s' password='%(password)s' port='%(port)s'" % parameters)
        self.conn.autocommit = True

    def __del__(self):
        self.conn.close()



import time


class Counter(object):
    """Time counter."""

    def __init__(self):
        """Sets start"""
        self._start = time.time()

    def begin(self):
        """Overwrites start"""
        self._start = time.time()

    def miliseconds(self):
        """Number of seconds from start"""
        return int(round((time.time() - self._start) * 1000))

    def as_string(self):
        """Number of seconds from start"""

        seconds = self.miliseconds() / 1000

        m, s = divmod(seconds, 60)
        h, m = divmod(m, 60)
        return "%d:%02d:%02d" % (h, m, s)


class Singleton:
    """
    A non-thread-safe helper class to ease implementing singletons.
    This should be used as a decorator -- not a metaclass -- to the
    class that should be a singleton.

    The decorated class can define one `__init__` function that
    takes only the `self` argument. Other than that, there are
    no restrictions that apply to the decorated class.

    To get the singleton instance, use the `Instance` method. Trying
    to use `__call__` will result in a `TypeError` being raised.

    Limitations: The decorated class cannot be inherited from.

    """

    def __init__(self, decorated):
        self._decorated = decorated

    def Instance(self):
        """
        Returns the singleton instance. Upon its first call, it creates a
        new instance of the decorated class and calls its `__init__` method.
        On all subsequent calls, the already created instance is returned.

        """
        try:
            return self._instance
        except AttributeError:
            self._instance = self._decorated()
            return self._instance

    def __call__(self):
        raise TypeError('Singletons must be accessed through `Instance()`.')

    def __instancecheck__(self, inst):
        return isinstance(inst, self._decorated)


#!/usr/bin/env python

from setuptools import setup, find_packages

def readme():
    with open("README.rst") as f:
        return f.read()

setup(
    name="sparweltbitool",
    version='0.1.1',
    long_description=readme(),
    summary="Python library with standard modules used by Sparwelt GmbH BI Team in Web Services.",
    author="Paweł Graczyk",
    author_email='abs-saas@sparwelt.de',
    py_modules=['sparweltbitool'],
    url="https://github.com/sparwelt/bitool.git",
    packages=find_packages(exclude=['tests*']),
    license='MIT',
    include_package_data=True,
    platform="python",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



from boto.s3.connection import S3Connection
import boto
from sparweltbitool.config import config
from sparweltbitool.logger import Logger

class S3Client():
    """
    Operations with s3 bucket.
    Check documentation: http://boto.readthedocs.org/en/latest/ref/s3.html
    """

    def __init__(self):
        self.conn = S3Connection(config.get('aws', 'access_key_id'), config.get('aws', 'secret_access_key'))
        self.logger = Logger()

    def send_file_local(self, key, file_path_local):
        """ Fetch all  files for current user."""
        conn = self.conn

        message = "Sending local file: '{}' under a key: '{}' on s3 bucket: '{}' set on region: '{}'".format(
            file_path_local,
            key,
            config.get('aws', 'bucket'),
            config.get('aws', 'region'))

        Logger().debug(message)

        bucket = conn.get_bucket(config.get('aws', 'bucket'))
        if not bucket.get_location():
            conn = boto.s3.connect_to_region(config.get('aws', 'region'))
            bucket = conn.get_bucket(config.get('aws', 'bucket'))

        return bucket.new_key(key).set_contents_from_filename(file_path_local)



import os
import logging
from sparweltbitool.config import config

class Logger(object):
    """Configuration for logging."""

    def __init__(self, identifier=None):

        logger_identifier = identifier if identifier is not None else config.get('logger', 'default_identifier')

        self.logger = logging.getLogger(logger_identifier)
        logging.basicConfig(
            filename=os.getcwd() + os.path.normpath(config.get('logger', 'log_file')),
            level=config.get('logger', 'log_level'),
            format='[%(asctime)-15s] {}.%(levelname)s: %(message)s'.format(config.get('logger', 'app_name')),
            datefmt='%Y-%m-%d %H:%M:%S'
        )

    def error(self, msg):
        logging.error(msg)

    def warning(self, msg):
        logging.warning(msg)

    def info(self, msg):
        logging.info(msg)

    def debug(self, msg):
        logging.debug(msg)
